mod brc;
mod commandline;
mod initialize;
mod search;

use anyhow::Result;
use commandline::{Cli, Commands};
use env_logger::Builder;
use initialize::init;
use log::{debug, LevelFilter};

use search::search;

fn main() -> Result<()> {
    let cli = <Cli as clap::Parser>::parse();

    // initial logging
    let max_log_level = match &cli.verbose {
        true => LevelFilter::Trace,
        false => LevelFilter::Info,
    };
    Builder::new().filter(None, max_log_level).init();
    debug!("Parsing subcommand");
    match &cli.command {
        Commands::Init {
            filename_option,
            key_option,
            index_option,
        } => {
            init(
                filename_option.clone(),
                key_option.clone(),
                *index_option,
                cli.address.clone(),
            )?;
        }
        Commands::Search {
            min,
            max,
            urf,
            output_path,
        } => {
            search(
                *min,
                *max,
                cli.address.clone(),
                *urf,
                output_path.to_owned(),
            )?;
        }
    }
    Ok(())
}
