use std::collections::HashMap;
use std::io::prelude::*;

use anyhow::Result;

// This is just a current implementation of the outputs
use crypto_utils::chacha::rand::SeedableRng;
use log::info;
use url::Url;

use crypto_utils::chacha::decrypt;
use crypto_utils::chacha::rand;
use crypto_utils::chacha::ChaCha20Rng;
use crypto_utils::hash::prf;
use crypto_utils::keys::load_from_file;

use rand::seq::SliceRandom;

use crate::brc::brc_tree;

/// Search function
pub fn search(
    min: isize,
    max: isize,
    address: String,
    urf: bool,
    output_path: String,
) -> Result<()> {
    let enc_key = load_from_file("enc.key")?;
    let prf_key = load_from_file("prf.key")?;
    let mut burl = Url::parse(&address)?;
    let btree = brc_tree::parse_from_file(std::path::Path::new("brc_table.partitions"))?;
    // load covers from brc tree
    let mut covers = btree.covers(min, max);
    if urf {
        covers.sort_by_key(|a| a.level());
        covers = brc_tree::brc_to_urc(covers);
    }
    info!(
        "Best range covers with height: {:?}",
        covers.iter().map(|x| x.range_level()).collect::<Vec<_>>()
    );
    let search_keys: Result<Vec<String>> = covers
        .iter()
        .map(|x| prf(&prf_key, format!("{:?}", x.range())))
        .collect();
    let mut keys = search_keys?;
    keys.shuffle(&mut ChaCha20Rng::from_os_rng());
    let mut fnl: Vec<String> = Vec::new();
    for key in keys {
        info!("Searching for key {}", key);
        burl.set_path(&format!("key/{}", key));
        let resp = reqwest::blocking::get(burl.clone())?.json::<HashMap<String, String>>()?;
        for (_, ciphertext) in resp.iter() {
            let decrypted = decrypt(&enc_key, ciphertext.to_string())?;
            fnl.push(decrypted);
        }
    }
    info!("Found {} results", fnl.len());
    let mut out_file = std::fs::File::create(output_path)?;
    out_file.write_all(fnl.join("\n").as_bytes())?;
    Ok(())
}
