use clap::{Parser, Subcommand};
use url::Url;

#[derive(Parser)]
#[command(version, about, long_about=None)]
pub struct Cli {
    /// Verbose level (Trace / Info)
    #[clap(global = true)]
    #[arg(short, long, default_value_t = false)]
    pub verbose: bool,

    /// Subcommands
    #[command(subcommand)]
    pub command: Commands,

    #[clap(global = true)]
    #[arg(short, long, num_args = 1.., default_values = ["http://localhost:8080/"])]
    pub address: Vec<Url>,
}

#[derive(Subcommand)]
pub enum Commands {
    /// Initializes the remote server with file
    Init {
        /// File path to be inputted
        filename_option: String,
        /// set searchable column to key
        #[clap(long = "key", short = 'k')]
        key_option: Option<String>,
        /// set searchable index
        #[clap(long = "index", short = 'i')]
        index_option: Option<usize>,
    },
    /// Searches remote server with optional string
    Search {
        /// Minimum value in range (inclusive)
        min: isize,
        /// Maximum in range (inclusive)
        max: isize,
        /// Optional urf option
        #[clap(long = "urf", short = 'u', default_value_t = false)]
        urf: bool,

        #[arg(short, long, default_value = "out.csv")]
        output_path: String,
    },
    /// Adds to remote key-value store
    Insert {
        /// Value to add, must be in a csv row format that fits the given scheme
        value: String,
    },
    Delete {
        /// Key to delete
        key: String,
    },
}
