mod brc;
mod commandline;
mod initialize;
mod insert;
mod pools;
mod search;
use anyhow::Result;
use commandline::{Cli, Commands};
use crypto_utils::chacha::{rand::SeedableRng, ChaCha20Rng};
use env_logger::Builder;
use initialize::init;
use insert::{delete, insert};
use log::{debug, LevelFilter};

use search::search;

#[tokio::main]
async fn main() -> Result<()> {
    let cli = <Cli as clap::Parser>::parse();

    // initial logging
    let max_log_level = match &cli.verbose {
        true => LevelFilter::Trace,
        false => LevelFilter::Error,
    };
    Builder::new().filter(None, max_log_level).init();
    debug!("Parsing subcommand");
    let mut rng = ChaCha20Rng::from_os_rng();
    match &cli.command {
        Commands::Init {
            filename_option,
            key_option,
            index_option,
        } => {
            init(
                filename_option.clone(),
                key_option.clone(),
                *index_option,
                cli.address,
                &mut rng,
            )
            .await?;
        }
        Commands::Search {
            min,
            max,
            urf,
            output_path,
        } => {
            search(
                *min,
                *max,
                cli.address,
                *urf,
                output_path.to_owned(),
                &mut rng,
            )
            .await?;
        }
        Commands::Insert { value } => {
            insert(value.to_owned(), cli.address.clone(), &mut rng).await?;
        }
        Commands::Delete { key } => {
            delete(key.to_owned(), cli.address.clone(), &mut rng).await?;
        }
    }
    Ok(())
}
