use std::collections::HashMap;

use anyhow::{anyhow, Result};

use crypto_utils::chacha::{rand::seq::IndexedRandom, ChaCha20Rng};
use serde::{Deserialize, Serialize};
use url::Url;

#[derive(Serialize, Deserialize, Debug)]
pub struct MetaData {
    pub index: i32,
    pub header: Vec<String>,
    pub row_length: i32,
}

pub async fn search_availability(server_list: Vec<Url>) -> Result<HashMap<Url, Vec<String>>> {
    let mut futures = Vec::new();
    for server_url in server_list {
        futures.push(async move {
            let client = reqwest::Client::new();
            let mut path = server_url.clone();
            path.set_path("/btrees");
            let res = client
                .get(path.to_owned())
                .send()
                .await?
                .json::<Vec<String>>()
                .await?;
            Ok::<(Url, Vec<String>), reqwest::Error>((server_url, res))
        })
    }
    Ok(futures::future::join_all(futures)
        .await
        .into_iter()
        .collect::<Result<Vec<(Url, Vec<String>)>, reqwest::Error>>()?
        .into_iter()
        .collect())
}
pub async fn post_metadata(server_list: Vec<Url>, content: String) -> Result<()> {
    let mut futures = Vec::new();
    for server_url in server_list {
        let content = content.clone();
        futures.push(async move {
            let client = reqwest::Client::new();
            let mut path = server_url.clone();
            path.set_path("/metadata");
            println!("senidng {:?}", content);
            client.post(path).body(content.clone()).send().await?;
            Ok::<(), reqwest::Error>(())
        })
    }
    futures::future::join_all(futures).await;
    Ok(())
}
pub async fn pool_get_metadata(server_list: Vec<Url>, rng: &mut ChaCha20Rng) -> Result<String> {
    let mut metadata = server_list
        .choose(rng)
        .ok_or(anyhow!("no available servers"))?
        .to_owned();
    metadata.set_path("/metadata");
    Ok(reqwest::Client::new()
        .get(metadata)
        .send()
        .await?
        .text()
        .await?)
}

pub async fn first_available_sda_server(
    sda_level: usize,
    urls: Vec<Url>,
    rng: &mut ChaCha20Rng,
) -> Result<Url> {
    // Ok(sda_availability(sda_level, urls.clone())
    //     .await?
    //     .choose(rng)
    //     .or(urls.choose(&mut ChaCha20Rng::from_os_rng()))
    //     .ok_or(anyhow!("No sda servers available"))?
    //     .to_owned())
    let _ = sda_level;
    urls.choose(rng)
        .ok_or(anyhow!("No sda servers available"))
        .map(|x| x.to_owned())
}
