use clap::{Parser, Subcommand};

#[derive(Parser)]
#[command(version, about, long_about=None)]
pub struct Cli {
    /// Verbose level (Trace / Info)
    #[arg(short, long, default_value_t = false)]
    pub verbose: bool,

    /// Subcommands
    #[command(subcommand)]
    pub command: Commands,

    #[clap(global = true)]
    #[arg(short, long, default_value = "http://localhost:8080/")]
    pub address: String,
}

#[derive(Subcommand)]
pub enum Commands {
    /// Initializes the remote server with file
    Init {
        /// File path to be inputted
        filename_option: String,
        /// set searchable column to key
        #[clap(long = "key", short = 'k')]
        key_option: Option<String>,
        /// set searchable index
        #[clap(long = "index", short = 'i')]
        index_option: Option<usize>,
    },
    /// Searches remote server with optional string
    Search { phrase: String },
}
