use anyhow::Result;

mod initialize;
use initialize::init;

mod search;
use search::search;

mod commandline;
use commandline::{Cli, Commands};

use env_logger::Builder;
use log::{debug, LevelFilter};

fn main() -> Result<()> {
    let cli = <Cli as clap::Parser>::parse();

    // initial logging
    let max_log_level = match &cli.verbose {
        true => LevelFilter::Trace,
        false => LevelFilter::Info,
    };
    Builder::new().filter(None, max_log_level).init();

    debug!("Parsing subcommand");
    match &cli.command {
        Commands::Init {
            filename_option,
            key_option,
            index_option,
        } => {
            init(
                filename_option.clone(),
                key_option.clone(),
                *index_option,
                cli.address.clone(),
            )?;
        }
        Commands::Search { phrase } => {
            search(phrase.clone(), cli.address.clone())?;
        }
    }
    Ok(())
}
