import csv

from faker import Faker

## TO USE:
## python3 generator.py <seed> <num_rows>


def generate(seed=0, num_rows=2**20):
    fake = Faker()
    Faker.seed(seed)

    col_headers = ["Name", "Email", "Country", "Age", "Salary", "Years of Experience"]
    
    data = {
        "Name": [fake.name() for _ in range(num_rows)],
        "Email": [fake.email() for _ in range(num_rows)],
        "Country": [fake.country() for _ in range(num_rows)],
        "Age": [fake.random_int(min=18, max=65) for _ in range(num_rows)],
        "Salary": [fake.random_int(min=37, max=150) * 1000 for _ in range(num_rows)],
        "Years of Experience": [
            fake.random_int(min=0, max=15) for _ in range(num_rows)
        ],
    }

    with open(f"syn_data_{num_rows}.csv", "w") as file:
        writer = csv.DictWriter(file, fieldnames=col_headers)

        writer.writeheader()

        for row in zip(*data.values()):
            writer.writerow(dict(zip(col_headers, row)))


if __name__ == "__main__":
    from sys import argv

    if len(argv) == 3:
        generate(int(argv[1]), int(argv[2]))
    else:
        generate()
