import csv
import glob
import json
import os
import subprocess
import time

import matplotlib.pyplot as plt


def test_correctness():
    subprocess.run(["./client", "init", "syn_data_16384.csv", "--index", "4"])
    subprocess.run(["./client", "search", "31000", "55000"])
    # check whether the output is the same
    expected = []
    with open("syn_data_16384.csv") as f:
        reader = csv.DictReader(f)
        for row in reader:
            if 31000 <= int(row["Salary"]) <= 55000:
                expected.append(",".join(row.values()))
    expected = list(set(expected))
    with open("out.csv", "r") as f:
        reader = csv.DictReader(f)
        exists = []
        for row in reader:
            cur = ",".join(row.values())
            exists.append(cur)
            assert cur in expected
        assert len(exists) == len(expected)
    subprocess.run(["./client", "search", "31000", "55000", "-u"])
    with open("out.csv", "r") as f:
        reader = csv.DictReader(f)
        exists = []
        for row in reader:
            cur = ",".join(row.values())
            exists.append(cur)
            assert cur in expected
        assert len(exists) == len(expected)
    subprocess.run(
        [
            "./client",
            "insert",
            "Russell Reynolds,crobinson@example.com,Armenia,64,45000,10",
        ]
    )
    expected.append("Russell Reynolds,crobinson@example.com,Armenia,64,45000,10")
    expected = list(set(expected))
    subprocess.run(["./client", "search", "31000", "55000", "-u"])
    with open("out.csv", "r") as f:
        reader = csv.DictReader(f)
        exists = []
        for row in reader:
            cur = ",".join(row.values())
            exists.append(cur)
            assert cur in expected
        with open("expected", "w") as f:
            f.write("\n".join(expected))
        with open("exists", "w") as f:
            f.write("\n".join(exists))
        assert len(exists) == len(expected), (
            f"{len(exists)} != {len(expected)}, {[x for x in exists if x not in expected]} + {[x for x in expected if x not in exists]}"
        )
    time.sleep(1)
    subprocess.run(["./client", "delete", "45000"])
    expected = [x for x in expected if x.split(",")[4] != "45000"]
    subprocess.run(["./client", "search", "31000", "55000", "-u"])
    with open("out.csv", "r") as f:
        reader = csv.DictReader(f)
        exists = []
        for row in reader:
            cur = ",".join(row.values())
            exists.append(cur)
            assert cur in expected, f"{cur} not in expected"
        assert len(exists) == len(expected), (
            f"{len(exists)} != {len(expected)}, {[x for x in exists if x not in expected]}"
        )
    with open("syn_data_1024.csv") as f:
        reader = csv.DictReader(f)
        for row in reader:
            ins_string = ",".join(row.values())
            if 31000 <= int(row["Salary"]) <= 55000:
                expected.append(ins_string)
            subprocess.run(["./client", "insert", ins_string])
    time.sleep(1)
    subprocess.run(["./client", "search", "31000", "55000", "-u"])
    with open("out.csv", "r") as f:
        reader = csv.DictReader(f)
        exists = []
        for row in reader:
            cur = ",".join(row.values())
            exists.append(cur)
            assert cur in expected, f"{cur} not in expected"
        assert len(exists) == len(expected)


# test_correctness()


def size_difference():
    files = [
        "syn_data_1024.csv",
        "syn_data_8192.csv",
        "syn_data_16384.csv",
        "syn_data_65536.csv",
        "syn_data_131072.csv",
        "syn_data_524288.csv",
        "syn_data_1048576.csv",
    ]
    sizes = {}
    size_ratio = {}
    times = {}
    for file in files:
        print(file)
        start = time.perf_counter()
        subprocess.run(["./client", "init", file, "--index", "5"])
        end = time.perf_counter()
        newfile = glob.glob("../server/*.json")[0]
        sizes[file] = os.path.getsize(newfile)
        times[file] = end - start
        size_ratio[file] = sizes[file] / os.path.getsize(file)
        os.remove(newfile)
    print(sizes)
    print(size_ratio)
    print(times)


# size_difference()
def update_times():
    results = {}
    with open("syn_data_16384.csv", "r") as f:
        count = 0
        reader = csv.DictReader(f)
        for row in reader:
            ins_string = ",".join(row.values())
            start = time.perf_counter()
            subprocess.run(["./client", "insert", ins_string])
            end = time.perf_counter()
            print(f"{count}: {end - start}")
            results[count] = end - start
            count += 1
    print(results)
    with open("results.json", "w") as f:
        json.dump(results, f)


def check():
    expect = []
    with open("syn_data_16384.csv", "r") as f:
        reader = csv.DictReader(f)
        for row in reader:
            if 3 <= int(row["Years of Experience"]) <= 6:
                expect.append(",".join(row.values()))
    expect = list(set(expect))
    subprocess.run(["./client", "search", "3", "6", "-u"])
    with open("out.csv", "r") as f:
        reader = csv.DictReader(f)
        exists = []
        for row in reader:
            cur = ",".join(row.values())
            exists.append(cur)
            assert cur in expect
        assert len(exists) == len(expect), (
            f"{len(exists)} != {len(expect)}, {len([x for x in exists if x not in expect]) + len([x for x in expect if x not in exists])}"
        )


# check()
#
def visualize():
    with open("results.json", "r") as f:
        results = json.load(f)

    plt.ylabel("Time (s)")
    plt.plot(results.keys(), results.values())
    plt.show()


def visualize_brc():
    plt.title("BRC")
    with open("results_brc.json", "r") as f:
        results = json.load(f)
    plt.ylabel("Time (s)")
    plt.plot(results.keys(), results.values())
    plt.show()


# visualize_brc()


def average():
    with open("results.json", "r") as f:
        results = json.load(f)
    print(sum(results.values()) / len(results))


def search_time():
    results_brc = {}
    results_urc = {}
    with open("syn_data_1048576.csv", "r") as f:
        reader = csv.DictReader(f)
        values = {i: [] for i in range(0, 16)}
        for row in reader:
            ins_string = ",".join(row.values())
            values[int(list(row.values())[-1])].append(ins_string)
    subprocess.run(["./client", "init", "syn_data_1048576.csv", "--index", "5"])
    for i in [0, 1, 3, 7, 15]:
        expected = []
        for j in range(0, i + 1):
            expected += values[j]
        expected.sort()
        start = time.perf_counter()
        subprocess.run(["./client", "search", "0", f"{i}"])
        end = time.perf_counter()
        results_brc[i] = end - start
        with open("out.csv", "r") as f:
            reader = csv.DictReader(f)
            exists = []
            for row in reader:
                cur = ",".join(row.values())
                exists.append(cur)
            exists.sort()
            assert exists == expected, (
                f"{len(exists)} != {len(expected)}, {len([x for x in exists if x not in expected]) + len([x for x in expected if x not in exists])}"
            )
        start = time.perf_counter()
        subprocess.run(["./client", "search", "0", f"{i}", "-u"])
        end = time.perf_counter()
        results_urc[i] = end - start
        with open("out.csv", "r") as f:
            reader = csv.DictReader(f)
            exists = []
            for row in reader:
                cur = ",".join(row.values())
                exists.append(cur)
            exists.sort()
            assert exists == expected, (
                f"{len(exists)} != {len(expected)}, {len([x for x in exists if x not in expected]) + len([x for x in expected if x not in exists])}"
            )
        with open("results_brc.json", "w") as f:
            json.dump(results_brc, f)
        with open("results_urc.json", "w") as f:
            json.dump(results_urc, f)


# search_time()
# average()
# visualize()
# update_times()
# size_difference()
